
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/times.h>
#include <unistd.h>     /* _SC_CLK_TCK */

#include <string.h>
#include <error.h>
#include <locale.h>
#include <math.h>

static char const *input_file = NULL;
static char const *output_file = NULL;
static size_t input_blocksize = 512;
static size_t output_blocksize = 512;

int input_flags = 0;
int output_flags = 0;


static void
readopt (int argc, char **argv)
{
	int i;
	int bs=0;
	char *name;
	char *val;

	for (i = optind; i < argc; i++) {

		name = argv[i];
		val = strchr (name, '=');


		if (val == NULL) {
			error (0, 0, ("незарегистрированная опция %s"), name);
			exit (EXIT_FAILURE);
		}

		*val++ = '\0';

		if (strcmp(name,"if") == 0) {
			input_file = val;
			input_flags = 1;

		} else if (strcmp(name,"of") == 0) {
			output_file = val;
			output_flags = 1;

		} else if (strcmp(name,"ibs") == 0) {
			if (bs == 0) 
				input_blocksize = atof(val);

		} else if (strcmp(name,"obs") == 0) {
			if (bs == 0) 
				output_blocksize = atof(val);

		} else if (strcmp(name,"bs") == 0) {
			input_blocksize = atof(val);
			output_blocksize = atof(val);
			bs = 1;
		}
	}
}

int main(int argc, char *argv[]) 
{
	setlocale (LC_ALL, "");
	FILE *in, *out;
	struct tms tms_stop,  tms_start;
	clock_t    real_stop, real_start, HZ;

	int  n,i,y;
	int col = 0;
	int col2 = 0;
	int col3 = 0;
	int col4 = 0;

	struct stat st;

	int ost=0; /* кол-во байт которые остались незаписанными после записи предыдущего блока */

	readopt(argc, argv); /* определение опций со знаком =*/

	char bufin[input_blocksize];
	char bufout[output_blocksize];

	if ((input_flags == 0) || (output_flags == 0)) {
		printf("ошибка\n");
		return EXIT_SUCCESS;
	}

	if ((in = fopen(input_file, "rb")) == NULL) {
		printf("Невозможно прочитать файл");
	}
	
	out = fopen(output_file, "w");

	HZ = sysconf(_SC_CLK_TCK);
	real_start = times(&tms_start);

/* копирование если блок чтения равняется блоку записи */
	if (input_blocksize == output_blocksize) {
		while ((n = read(fileno(in), bufin, input_blocksize)) > 0) {
			write(fileno(out), bufin, n);
			col++;
			col2 = col;
			y = n;
		}

		if (y < input_blocksize) {
			col--;
			col2--;
			col3++;
			col4++;	
		}
	}

/* копирование по блокам если блок чтения больше блока записи */
	if (input_blocksize > output_blocksize) {
		int raz2 = 0;
		int ot,to,k;

		while ((n = read(fileno(in),bufin, input_blocksize)) >0) {
			col++;
			k = n;

			/* Если остаток + полученные байты меньше размера блока необходимого для записи */
			if (ost+n < output_blocksize) {
				for (y = 0; y < n; y++) {
					bufout[y+ost] = bufin[y];
				};
				write(fileno(out),bufout,n+ost);
				ost = 0;
				col4++;
				exit;
			}

			/* записываем остаток + недостающие байты*/
			to = output_blocksize-ost;
			for (y = 0; y < to; y++) {
				bufout[y+ost] = bufin[y];
			};
			write(fileno(out),bufout,output_blocksize);
			col2++;


			/* узнаём во сколько раз размер прочитанного больше размера блока записи */
			raz2 = trunc((n-(output_blocksize-ost))/output_blocksize);
			
			/* записываем целые блоки */
			for (i = 0; i < raz2; i++) {
				ot = i*output_blocksize + (output_blocksize - ost);
				to = (i+1)*output_blocksize+ost+(output_blocksize - ost);

				for (y = ot; y < to; y++) {
					bufout[y-ot] = bufin[y];
				};
				
				write(fileno(out),bufout,output_blocksize);
				col2++;
			}

			/* записываем остаток в буфер */
			ot = raz2*output_blocksize+(output_blocksize-ost);
			to = n;

			for (y = ot; y < to; y++)  {
				bufout[y-ot] = bufin[y];
			}
			ost = n-ot;
		}

		if (ost > 0) {
			write(fileno(out),bufout,ost);
			col4++;
		}

		if (k < input_blocksize) {
			col--;
			col3++;
		}
	}
/* ========================================== конец ========================================*/

/* копирование по блокам если блок чтения меньше блока записи */

	if (input_blocksize < output_blocksize) {

		ost = 0;

		while ((n = read(fileno(in),bufin, input_blocksize)) >0) {
			col++;
			y = 0;

			for (i = 0; i < n; i++) {
				if (ost+1 < output_blocksize) {
					bufout[i+ost] = bufin[i];
					ost++;
				} else {
					bufout[i+ost] = bufin[i];
					write(fileno(out),bufout,output_blocksize);
					ost = 0;
					col2++;
				}
			}
		}
		
		if (ost != 0) {
			write(fileno(out),bufout,ost);
			col4++;
		}

		if (y < input_blocksize) {
			col--;
			col3++;
		}

	}
/* ========================================== конец ========================================*/

	real_stop = times(&tms_stop);

	fclose(out);
	fclose(in);

	stat(output_file, &st);
	printf("%u+%u записей считано \n", col, col3);
	printf("%u+%u записей написано \n", col2, col4);
	
	printf("скопировано: %d байт (%d B), %f c,  \n", st.st_size, st.st_size, (real_stop - real_start) / (double)HZ );

	return EXIT_SUCCESS;
}